unit Subst9x;

{ 
  API for work with substitution device (see dos command "subst"). 
  Win9x ONLY. For WinNT and Win2000 use native API call.
  Last rev. 2 Mar 2000

  Delphi 2,3,4,5
  Freware with source. 

  Copyright (c) 2000, SoftLab
  Web     http://softlab.od.ua/
  Email   support@softlab.od.ua

  THIS SOFTWARE AND THE ACCOMPANYING FILES ARE DISTRIBUTED 
  "AS IS" AND WITHOUT WARRANTIES AS TO PERFORMANCE OF MERCHANTABILITY OR 
  ANY OTHER WARRANTIES WHETHER EXPRESSED OR IMPLIED.
  NO WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE IS OFFERED.
  THE USER MUST ASSUME THE ENTIRE RISK OF USING THE ACCOMPANYING CODE.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions: 

  1. The origin of this software must not be misrepresented, you must 
     not claim that you wrote the original software. If you use this software 
     in a product, an acknowledgment in the product documentation 
     would be appreciated but is not required. 
  2. Altered source versions must be plainly marked as such, and must not be 
     misrepresented as being the original software.
  3. Original copyright may not be removed or altered from any source 
     distribution.
}

interface

uses Windows, SysUtils;

function SubstCreate(DriveLetter: Char; const Path: string): Boolean;
// create subst device. Path must be a real folder.

function SubstRemove(DriveLetter: Char): Boolean;
// destroy subst device.

function SubstQuery(DriveLetter: Char): string;
// get Path for subst device. 
// return empty string if DriveLetter is not subst-device

implementation

procedure VxDCall; external kernel32 index 1;

function SubstCreate(DriveLetter: Char; const Path: string): Boolean;
var drvno: byte;
    buff: array[0..1023] of char;
    FPath: string;
begin
  if Path[Length(Path)] = '\' then
    FPath:=Copy(Path,1,Length(Path)-1)
  else
    FPath:=Path;
  FPath:=ExtractShortPathName(FPath);
  if FPath <> '' then begin
    drvno:=Ord(UpperCase(DriveLetter)[1])-64;
    StrPCopy(buff,FPath);
    asm
       pushad
       push    es
       xor     ebx, ebx
       mov     bh,0
       mov     bl,drvno
       lea     edx,buff
       push    0       //ECX (unused)
       push    71AAh
       push    2A0010h
       call    VxDCall
       pop     es
       popad
    end;
  end;
  Result:=SubstQuery(DriveLetter) <> '';
end;

function SubstRemove(DriveLetter: Char): Boolean;
var drvno: byte;
begin
  drvno:=Ord(UpperCase(DriveLetter)[1])-64;
  asm
     pushad
     push    es
     xor     ebx, ebx
     mov     bh,1
     mov     bl,drvno
     push    0       //ECX (unused)
     push    71AAh
     push    2A0010h
     call    VxDCall
     pop     es
     popad
  end;
  Result:=SubstQuery(DriveLetter) = '';
end;

function SubstQuery(DriveLetter: Char): string;
var drvno: byte;
    buff: array[0..1023] of char;
begin
  drvno:=Ord(UpperCase(DriveLetter)[1])-64;
  buff[0]:=#0;
  asm
     pushad
     push    es
     xor     ebx, ebx
     mov     bh,2
     mov     bl,drvno
     lea     edx,buff
     push    0       //ECX (unused)
     push    71AAh
     push    2A0010h
     call    VxDCall
     pop     es
     popad
  end;
  Result:=StrPas(buff);
end;

end.
